@@ -1,23 +1,25 @@
-/*  GNU ddrescue - Data recovery tool
-    Copyright (C) 2013-2020 Antonio Diaz Diaz.
+/* GNU ddrescue - Data recovery tool
+   Copyright (C) 2013-2022 Antonio Diaz Diaz.
 
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
+   This program is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 2 of the License, or
+   (at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #define _FILE_OFFSET_BITS 64
 
+#include <algorithm>
 #include <cstdio>
+#include <cstring>
 #include <string>
 #include <vector>
 #include <sys/stat.h>
@@ -51,22 +53,24 @@ Rate_logger rate_logger;
 Read_logger read_logger;
 
 
-bool Logger::set_filename( const char * const name )
+void Logger::set_filename( const char * const name )
   {
-  if( name && name[0] )
-    {
-    struct stat st;
-    if( stat( name, &st ) == 0 && !S_ISREG( st.st_mode ) ) return false;
-    filename_ = name;
-    }
-  return true;
+  if( !name || !name[0] ) return;			// ignore name
+  if( std::strcmp( name, "-" ) == 0 )
+    { show_error( "I won't write log data to standard output." );
+      std::exit( 1 ); }
+  struct stat st;
+  if( stat( name, &st ) == 0 && !S_ISREG( st.st_mode ) )
+    { show_file_error( name, "Logfile exists and is not a regular file." );
+      std::exit( 1 ); }
+  filename_ = name;
   }
 
 
 bool Logger::close_file()
   {
   if( f && !error && !write_final_timestamp( f ) ) error = true;
-  if( f && std::fclose( f ) != 0 ) error = true;
+  if( f && std::fclose( f ) != 0 ) error = true;	// close even on error
   f = 0;
   return !error;
   }
